import envoy
import gleam/erlang/process
import gleam/int
import mist
import switcheroo/router
import switcheroo/web.{Context}
import wisp
import wisp/wisp_mist

pub fn main() -> Nil {
  wisp.configure_logger()

  let assert Ok(secret_key_base) = envoy.get("SECRET_KEY")
  let assert Ok(port_str) = envoy.get("PORT")
  let assert Ok(port) = int.parse(port_str)

  let ctx = Context(static_directory: static_directory())

  let assert Ok(_) =
    router.handle_request(_, ctx)
    |> wisp_mist.handler(secret_key_base)
    |> mist.new
    |> mist.bind("0.0.0.0")
    |> mist.port(port)
    |> mist.start_http

  process.sleep_forever()
}

pub fn static_directory() -> String {
  // The priv directory is where we store non-Gleam and non-Erlang files,
  // including static assets to be served.
  // This function returns an absolute path and works both in development and in
  // production after compilation.
  let assert Ok(priv_directory) = wisp.priv_directory("switcheroo")
  priv_directory <> "/static"
}
