import gleam/crypto
import gleam/http.{Delete, Get, Post}
import gleam/http/cookie
import gleam/http/response
import gleam/list
import gleam/option
import gleam/string_tree
import wisp.{type Request, type Response}

pub const cookie_name = "pluralkit_token"

pub fn login(req: Request) -> Response {
  case req.method {
    Get -> get_login(req)
    Post -> post_login(req)
    Delete -> delete_login(req)
    _ -> wisp.method_not_allowed([Get, Post, Delete])
  }
}

fn get_login(_req: Request) -> Response {
  [
    "<p>Hi, need that one's PluralKit token! <3</p>",
    "<form action='/login' method='post'>",
    "  <label for='token'><b>PluralKit Token</b></label>",
    "  <input type='string' name='token' id='token' />",
    "  <input type='submit' value='Login'/>", "</form>",
  ]
  |> string_tree.from_strings
  |> wisp.html_response(200)
}

fn post_login(req: Request) -> Response {
  use formdata <- wisp.require_form(req)

  case list.key_find(formdata.values, "token") {
    Ok(token) -> {
      echo "token: " <> token

      let value = wisp.sign_message(req, <<token:utf8>>, crypto.Sha512)
      let attrs =
        cookie.Attributes(
          ..cookie.defaults(http.Http),
          max_age: option.Some(60 * 60 * 24 * 365),
        )

      wisp.redirect("/")
      |> response.set_cookie(cookie_name, value, attrs)
    }
    Error(e) -> {
      echo e
      ["<p>awawa it didn't work</p>"]
      |> string_tree.from_strings
      |> wisp.html_response(400)
      |> wisp.set_header("x-servfail", "token not found in formdata")
    }
  }
}

fn delete_login(req: Request) -> Response {
  let resp = wisp.redirect("/")
  case wisp.get_cookie(req, cookie_name, wisp.PlainText) {
    Ok(value) ->
      wisp.set_cookie(resp, req, cookie_name, value, wisp.PlainText, 0)
    Error(_) -> resp
  }
}
