import gleam/list
import lustre/attribute.{class, id}
import lustre/element.{type Element, text}
import lustre/element/html

pub type Member {
  Member(name: String, uuid: String)
}

pub type PickerProps {
  PickerProps(members: List(Member))
}

pub fn picker(props: PickerProps) -> Element(PickerProps) {
  element.fragment([
    html.head([], [
      html.meta([attribute.charset("utf-8")]),
      html.meta([
        attribute.name("viewport"),
        attribute.content("width=device-width, initial-scale=1.0"),
      ]),
      html.script(
        [attribute.src("/static/picker.mjs"), attribute.type_("module")],
        "",
      ),
      // california style sheet
      html.link([
        attribute.rel("stylesheet"),
        attribute.href("/static/picker.css"),
      ]),
    ]),
    html.body([], [
      // bucket for the new ordering
      html.section([class("switch-bucket")], [
        html.h2([], [text("switching to:")]),
        html.p([id("switch-list")], []),
        picker_form(props.members),
      ]),
    ]),
    // buttons!!!
    // this should scroll separately
    html.section(
      [class("buttons")],
      props.members
        |> list.map(button),
    ),
  ])
}

fn button(member: Member) -> Element(PickerProps) {
  html.button(
    [
      // we do a little HATEOAS
      attribute.data("name", member.name),
      attribute.data("uuid", member.uuid),
      class("member-button"),
    ],
    [text(member.name)],
  )
}

fn picker_form(members: List(Member)) -> Element(PickerProps) {
  html.form([attribute.method("post"), attribute.action("")], [
    // JS will fill out this form via the buttons
    html.div([class("hidden"), attribute.aria_hidden(True)], [
      // only one fronter (as in first front)
      picker_form_set(members, "radio", "pf-fronter"),
      // many backups
      picker_form_set(members, "checkbox", "pf-backups"),
    ]),
    html.input([
      attribute.type_("submit"),
      attribute.value("switch"),
      id("submit"),
      // enable after first pick
      attribute.disabled(True),
    ]),
  ])
}

fn picker_form_set(
  members: List(Member),
  input_type: String,
  set_name: String,
) -> Element(PickerProps) {
  html.div(
    [],
    members
      |> list.map(fn(member: Member) -> Element(PickerProps) {
        element.fragment([
          html.input([
            attribute.type_(input_type),
            attribute.name(set_name),
            id(set_name <> "__" <> member.uuid),
            attribute.value(member.uuid),
          ]),
          html.label([attribute.for(set_name <> "__" <> member.uuid)], [
            text(member.name),
          ]),
        ])
      }),
  )
}
